#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   gridEditor
#
#       grid editorを起動する
#
#       使い方
#       gridEditor.py <caseDir> <timeFolder> <polyMeshFolder>
#       例：gridEditor.py ~/cavity 0 constant/polyMesh
#
#   11/08/11    新規作成
#   12/01/21    バグ修正
#

import os
import os.path
import sys
import glob
import treefoam
import pyTreeFoam
import getFoamContents
import editBoundaryConditionsDialog
import selectFieldsForEditDialog

nTreeFoam = "-1"        #ここから起動したgridEditorのnTeeFoamは「-1」
TreeFoamPath = ""
caseDir = ""
editDir = ""            #fieldDir
polyMeshDir = ""


#
#  showGridEditor
#
def showGridEditor(editDir, polyMeshDir):
    foldersFiles = pyTreeFoam.getFoldersFiles(editDir)
    fields = foldersFiles[1]

    #Grid編集
    if len(fields) > 0:
        #title, dirを設定
        caseFolder = editDir.split("/")[-2]
        timeFolder = editDir.split("/")[-1]
        title = ": " + caseFolder + "/" + timeFolder
        try:
            editBoundaryConditionsDialog.Show(nTreeFoam, title, polyMeshDir,
                    editDir, fields)
        except:
            mess = u"error:オープンできないfile（field or boundary）があります。"
            print mess
    else:
        mess = u"error:Fieldが選択されていません。\n  Fieldを選択してください。"
        print mess


#  getInstallFolder
#    TreeFoamのインストールpathを取得
def getInstallFolder():
    homeFolder = os.path.expanduser("~")
    fileName = homeFolder + "/.treeFoamInstallFolder"
    contents = getFileContents(fileName)[0]
    return contents

#  getFileContents
#    ファイル内容を読み込む
def getFileContents(fileName):
    f=open(fileName); contents=f.readlines(); f.close()
    return contents

#  setFileContents
#    ファイル内容を書き込む
def setFileContents(fileName, contents):
    f=open(fileName, "w")
    for line in contents:
        f.write(line)
    f.close()

#  checkDir
#    入力されたDirのチェック
def checkDir(a1, a2, a3):
    global caseDir, editDir, polyMeshDir
    error = ""
    caseDir = os.path.abspath(a1)
    editDir = os.path.abspath(a2)
    polyMeshDir = os.path.abspath(a3)
    if len(glob.glob(a1)) == 0:
        error = "指定されたcaseDirは、存在していません。\n " + caseDir
    elif len(glob.glob(a2)) == 0:
        error = "指定されたeditDirは、存在していません。\n " + editDir
    elif len(glob.glob(a3)) == 0:
        error = "指定されたpolyMeshDirは、存在しません。\n " + polyMeshDir
    if error != "":
        error = "error:" + error + "\n"
    return error


#
#  メイン
#  ------
#

#変数クリア
error = ""
a1=""
a2=""
a3=""

#TreeFoamのインストールpath取得
TreeFoamPath = getInstallFolder()

#commnad取得
comm = ""
if len(sys.argv) == 1:
    comm = "noDir"
elif len(sys.argv) == 2:
    comm = "caseDir"
elif len(sys.argv) == 3:
    comm = "caseEditDir"
elif len(sys.argv) == 4:
    comm = "fullDir"
else:
    comm = "error"
if len(sys.argv) > 1:
    if sys.argv[1] == "-h" or sys.argv[1] == "--help" or sys.argv[1] == "-help":
        comm = "help"

#各コマンドの実行
if comm == "help":
    error = "使い方\n"
    error += "  gridEditor <caseDir> <fieldFolder> <polyMeshFolder>\n"
    error += "  gridEditor <caseDir> <fieldFolder>\n"
    error += "  gridEditor <caseDir>\n"
    error += "  gridEditor\n"
    error += "  Ex: gridEditor ~/cavity 0 constant/polyMesh\n"
    error += "  アーギュメントを省略した場合は、currentDirがcaseDirになる。\n"

elif comm == "noDir":
    a1 = "."
    a2 = "0"
    a3 = "constant/polyMesh"

elif comm == "caseDir":
    a1 = sys.argv[1]
    a2 = "0"
    a3 = "constant/polyMesh"

elif comm == "caseEditDir":
    a1 = sys.argv[1]
    a2 = sys.argv[2]
    if len(a2.split("/")) == 2:
        folder = a2.split("/")[1]
        a3 = "constant/" + folder + "/polyMesh"
    else:
        a3 = "constant/polyMesh"

elif comm == "fullDir":
    a1 = sys.argv[1]
    a2 = sys.argv[2]
    a3 = sys.argv[3]
else:
    error = "error:アーギュメントの数が合いません。\n gridEditor <caseDir> <fieldFolder> <polyMeshFolder>\n Ex: gridEditor ~/cavity 0 constant/polyMesh"


if error != "":
    print "\n" + error
else:
    #dirのチェック
    error = checkDir(a1, a2, a3)
    if error != "":
        print "\n" + error
    else:
        #gridRditor起動
        showGridEditor(editDir, polyMeshDir)

