#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  getFoamContents
#      Foam形式のデータを取得する
#
#       10/03/04    新規作成
#       11/11/05    cellZoneの取得変更（1行で記述されている場合を追加）
#

#
#   controlDict(CaseDir, items)
#           内容の読込み
#   PatchNameFronBoundary(polyMeshDir)
#           patch name の読込み
#

import os
import re
import glob
import string

#
#  getSameLocationPatches
#    同じ位置のpatch名を取得
def getSameLocationPatches(polyMeshDirReg0, polyMeshDirReg1):
    minMaxP = []
    minMaxPN = []
    print "patchのmin,max座標を取得中..."
    minMaxP = getPatchesMinMaxLocations(polyMeshDirReg0)
    minMaxPN = getPatchesMinMaxLocations(polyMeshDirReg1)
    samePatch = []
    for patchContP in minMaxP:
        for patchContPN in minMaxPN:
            if checkSameLocationPatch(patchContP, patchContPN)==True:
                samePatch.append([patchContP[0], patchContPN[0]])
                print "   " + patchContP[0] + ", " + patchContPN[0] + " は、同じ位置です"
    return samePatch

#
#  checkSameLocationPatch
#    同じ位置のpatchかどうかチェックする
#    入力：[patchName, [minx, miny, minz], [maxx, maxy, maxz]]
def checkSameLocationPatch(patchCP, patchCPN):
    minPx = patchCP[1][0]
    minPy = patchCP[1][1]
    minPz = patchCP[1][2]
    maxPx = patchCP[2][0]
    maxPy = patchCP[2][1]
    maxPz = patchCP[2][2]
    minPNx = patchCPN[1][0]
    minPNy = patchCPN[1][1]
    minPNz = patchCPN[1][2]
    maxPNx = patchCPN[2][0]
    maxPNy = patchCPN[2][1]
    maxPNz = patchCPN[2][2]
    length = ((maxPx-minPx)**2 + (maxPy-minPy)**2 + (maxPz-minPz)**2)**0.5
    tolerance = abs(length / 100.0)
    if (abs(minPx-minPNx) < tolerance and
        abs(minPy-minPNy) < tolerance and
        abs(minPz-minPNz) < tolerance and
        abs(maxPx-maxPNx) < tolerance and
        abs(maxPy-maxPNy) < tolerance and
        abs(maxPz-maxPNz) < tolerance):
        flag = True
    else:
        flag = False
    return flag

#
#  getPatchesMinMaxLocations
#    patchの最大、最小値を取得して返す
#    戻り値：[[patchName, minP, maxP]]
def getPatchesMinMaxLocations(polyMeshDir):
    patchConts = getPatchCont(polyMeshDir)
    faceDir = polyMeshDir + "/faces"
    f=open(faceDir); faceContents=f.readlines(); f.close()
    pointDir = polyMeshDir + "/points"
    f=open(pointDir); pointContents=f.readlines(); f.close()
    minMaxP=[]
    for patch in patchConts:
        patchName = patch[0]
        nFaces = patch[1]
        startFace = patch[2]
        [minP, maxP] = getPatchMinMaxLocations(
                        faceContents, pointContents, startFace, nFaces)
        minMaxP.append([patchName, minP, maxP])
    return minMaxP

#
#  getPatchCont
#    boundaryからpatch内容を取得
#    戻り値：[[patchName, nFaces, startFace]]
def getPatchCont(polyMeshDir):
    boundDir = polyMeshDir + "/boundary"
    f=open(boundDir); contents=f.read(); f.close()
    p = skipFoamFileMem(contents, 0)
    #最初のpatch名を取得
    flag = 0
    [keyword, p] = getKeywordMem(contents, p)
    while keyword != "" and flag == 0:
        try:
            a=int(keyword)
            [keyword, p] = getKeywordMem(contents, p)
        except:
            flag = 1
    #patch名と内容をセット
    patchName = keyword
    [patchCont, p] = getMiddlePairMem(contents, p)
    ans = []
    while patchCont != "":
        nFaces = 0
        startFace = 0
        lines = patchCont.split("\n")
        for line in lines:
            cont = line.split(";")[0].split()
            if len(cont) >= 1:
                if cont[0] == "nFaces":
                    nFaces = int(cont[1])
                elif cont[0] == "startFace":
                    startFace = int(cont[1])
        patch = [patchName, nFaces, startFace]
        ans.append(patch)
        [[patchName, patchCont], p] = getNextKeywordAndContentsMem(contents, p)
    return ans

#
#  getPatchMinMaxLocation
#    patchのmin,max座標値を取得
def getPatchMinMaxLocations(faceContents, pointContents, startFace, nFaces):
    i=0
    while i<len(faceContents) and faceContents[i][0]!="(":    
        i+=1
    if i<len(faceContents):
        i+=1
        i = i+startFace
        ie = i+nFaces
        faceCont = faceContents[i:ie]
        #print faceCont
        pointNo = []
        for line in faceCont:
            [facePoints, p] = getSmallPairMem(line, 0)
            pNo = facePoints.split()
            pointNo.append(pNo)
        i=0
        while i<len(pointContents) and pointContents[i][0]!="(":
            i+=1
        if i<len(pointContents):
            i+=1
            pointCont = pointContents[i:]
            p = pointNo[0][0]
            pxyz = pointCont[int(p)]
            [point, p] = getSmallPairMem(pxyz, 0)
            point = point.split()
            minP = [float(point[0]), float(point[1]), float(point[2])]
            maxP = [float(point[0]), float(point[1]), float(point[2])]
            for point in pointNo:
                for p in point:
                    pxyz=pointCont[int(p)]
                    [point3, p] = getSmallPairMem(pxyz, 0)
                    point3 = point3.split()
                    px = float(point3[0])
                    py = float(point3[1])
                    pz = float(point3[2])
                    if minP[0] > px:
                        minP[0] = px
                    if minP[1] > py:
                        minP[1] = py
                    if minP[2] > pz:
                        minP[2] = pz
                    if maxP[0] < px:
                        maxP[0] = px
                    if maxP[1] < py:
                        maxP[1] = py
                    if maxP[2] < pz:
                        maxP[2] = pz
        else:
            print "「points」ファイルから座標が読み取れません。"
            minP=[]
            maxP=[]
    else:
        print "「faces」ファイルからpointNoが読み取れません。"
        minP=[]
        maxP=[]
    return [minP, maxP]


#
#  controlDict
#    conrolDictの内容を取得
def controlDict(wdir, items):
    fileName = wdir + "/controlDict"
    ans = []
    try:
        f = open(fileName)
        contents = f.readlines()
        f.close
    except:
        print "error: ", wdir, "は、OpenFOAMのCaseDirではありません!!"
        ans = []
        return
    i = 0
    contents = deleteComments(contents)
    for item in items:
        ans.append("")
        for line in contents:
            if re.search(item, line) != None:
                a=getValue(item, line)
                if a != "":
                    ans[i] = a
        i = i + 1
    return ans

#
#  getFoamFile
#    foamFileの内容を取得
#    入力     [path, fileName]
#    戻り値　　[[item, contents]]
def getFoamFile(wdir, fileName):
    work = wdir + "/" + fileName
    ans = []
    if glob.glob(work) != "":
        f = open(wdir + "/" + fileName)
        contents = getKeywordContents("FoamFile", f)
        f.close()
        lines = contents.split(";")
        for line in lines:
            a = line.split()
            if len(a) > 1:
                items = [a[0], a[1]]
                ans.append(items)
    else:
        print "error:",wdir,"/",fileName," は、存在しません。"
    return ans

#----------------文字列からの検索---------------
#
#  skipFoamFileMem
#    FoamFileをskipする
def skipFoamFileMem(strings, p):
    [dummy, p] = getKeywordContentsMem("FoamFile", strings, p)
    return p

#
#  getNextKeywordAndContentsMem
#    次のkeywordと内容を取得する
def getNextKeywordAndContentsMem(strings, p):
    [keyword, p] = getKeywordMem(strings, p)
    [contents, p] = getMiddlePairMem(strings, p)
    return [[keyword, contents], p]

#
#  getKeywordPointerMem
#    指定したkeywordのpointerを取得する（memory上で）
#　　（高速検索している為、keywordがコメント中でも、検索するの注意！）
#    合致しない場合は、戻り値「-1」になる。
def getKeywordPointerMem(keyword, strings, p):
    if p>=0 and p<len(strings):
        #高速検索のため、string.find()を使用してポインタを設定
        flag = 0
        while (p>=0) and (flag==0):
            pp = p
            p = string.find(strings[p:], keyword)
            if p>=0:
                p = pp + p
                #keyword前後の文字をチェック
                if (p-1 < 0):
                    stChara = strings[0]
                else:
                    stChara = strings[p-1]
                pe = p+len(keyword)
                if pe>=len(strings):
                    edChara = "\n"
                else:
                    edChara = strings[p+len(keyword)]
                if ((checkChara(stChara) == False) and
                    (checkChara(edChara) == False)):
                    flag = 1
                else:
                    p+=1
        return p

#
#  getKeywordContentsMem
#    指定したkeywordの内容を取得する（memory上で）
#    （高速検索している為、keywordがコメント中でも、検索するので注意）
def getKeywordContentsMem(keyword, strings, p):
    if p>=0 and p<len(strings):
        #高速検索のため、string.find()を使用してポインタを設定
        flag = 0
        while (p>=0) and (flag==0):
            pp = p
            p = string.find(strings[p:], keyword)
            if p>=0:
                p = pp + p
                #keyword前後の文字をチェック
                if (p-1 < 0):
                    stChara = strings[0]
                else:
                    stChara = strings[p-1]
                pe = p+len(keyword)
                if pe>=len(strings):
                    edChara = "\n"
                else:
                    edChara = strings[p+len(keyword)]
                if ((checkChara(stChara) == False) and
                    (checkChara(edChara) == False)):
                    flag = 1
                else:
                    p+=1
        #新しいポインタで再度検索し、データ取得
        if p>=0:
            [word, p] = getKeywordMem(strings, p)
            while (word != keyword) and (word != ""):
                [word, p] = getKeywordMem(strings, p)
            [contents, p] = getMiddlePairMem(strings, p)
        else:
            contents = ""
    else:
        contents = ""
    return [contents, p]

#
#  getKeywordMem
#    keywordを取得（memory上で）
def getKeywordMem(strings, p):
    #文字まで進む
    [chara, p] = get1charaMem(strings, p)
    while (checkChara(chara) != True) and (chara!=""):
        [chara, p] = get1charaMem(strings, p)
    keyWord = ""
    while (checkChara(chara) == True) and (chara!=""):
        keyWord += chara
        [chara, p] = get1charaMem(strings, p)
    return [keyWord, p]

#
#  getMiddlePairMem
#    { }内データを取得し返す（memory上で）
def getMiddlePairMem(strings, p):
    #"{"まで進む
    [chara, p] = get1charaMem(strings, p)
    while (chara != "{") and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
    #"}"まで読み込む
    count = 1
    contents = ""
    while (count > 0) and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
        if chara == "{":
            count = count + 1
        elif chara == "}":
            count = count - 1
        contents += chara
    if contents != "":
        if contents[-1] == "}":
            contents = contents[:-1]
    return [contents, p]

#
#  getSmallPairMem
#    ( )内データを取得し返す（memory上で）
def getSmallPairMem(strings, p):
    #"("まで進む
    [chara, p] = get1charaMem(strings, p)
    while (chara != "(") and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
    #")"まで読み込む
    count = 1
    contents = ""
    while (count > 0) and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
        if chara == "(":
            count = count + 1
        elif chara == ")":
            count = count - 1
        contents += chara
    if contents != "":
        if contents[-1] == ")":
            contents = contents[:-1]
    return [contents, p]

#
#  get1charaMem
#    get 1 character on memory
#    文字列（strings）とポインタ（p）を設定
def get1charaMem(strings, p):
    [chara, p] = get1c(strings, p)
    if chara == "/":
        [nextChara, p] = get1c(strings, p)
        if nextChara == "/":
            p = skipUntilCRMem(strings, p)      #行末までスキップ
            [chara, p] = get1c(strings, p)
        elif nextChara == "*":
            p= skipUntilSSMem(strings, p)       #"*/"までスキップ
            [chara, p] = get1c(strings, p)
        else:
            p-=1                                #1文字戻す
    return [chara, p]

#行末までスキップ
def skipUntilCRMem(strings, p):
    if p>=0 and p<len(strings):
        [chara, p] = get1c(strings, p)
        while (chara != "\n") and (chara != ""):
            [chara, p] = get1c(strings, p)
            #print chara," ",
        p-=1                                        #1文字戻す
    else:
        p=len(strings)
    return p
#*/までスキップ
def skipUntilSSMem(strings, p):
    if p>=0 and p<len(strings)-1:
        charas = strings[p:p+2]; p+=1
        while (charas != "*/") and (p<len(strings)-1):
            charas = strings[p:p+2]; p+=1
            #print charas, " ",
        p+=1
    else:
        p=len(strings)
    return p
#spスキップ
def skipSpaceMem(strings, p):
    if p>=0 and p<len(strings)-1:
        while strings[p] == " ":
            p+=1
    else:
        p=len(strings)
    return p
#1文字取得
def get1c(strings, p):
    if p>=0 and p<len(strings):
        chara = strings[p]; p +=1
    else:
        chara = ""; p = len(strings)
    return [chara, p]

#-------------ファイルからの読込で検索-------------
#
#  getKeywordContents
#    指定したkeywordの内容を取得する
def getKeywordContents(keyword, f):
    contents = ""
    word = getKeyword(f)
    while (word != keyword) and (word != ""):
        word = getKeyword(f)
    contents = getMiddlePair(f)
    return contents

#
#  getKeyword
#    keywordを取得
def getKeyword(f):
    #文字まで進む
    chara = get1chara(f)
    while (checkChara(chara) != True) and (chara!=""):
        chara = get1chara(f)
    keyWord = ""
    while (checkChara(chara) == True) and (chara!=""):
        keyWord += chara
        chara = get1chara(f)
    return keyWord

#
#  checkChara
#    keywordとして使える文字のチェック
def checkChara(chara):
    if chara > " " and chara <= "z":
        if chara == "{" or chara == "}":
            flag = False
        if chara == "(" or chara == ")":
            flag = False
        else:
            flag = True
    else:
        flag = False
    return flag

#
#  getMiddlePair
#    { }内データを取得し返す
def getMiddlePair(f):
    #"{"まで進む
    chara = get1chara(f)
    while (chara != "{") and (chara != ""):
        chara = get1chara(f)
    #"}"まで読み込む
    count = 1
    contents = ""
    while (count > 0) and (chara != ""):
        chara = get1chara(f)
        if chara == "{":
            count = count + 1
        elif chara == "}":
            count = count - 1
        contents += chara
    if contents[-1] == "}":
        contents = contents[:-1]
    return contents

#
#  get1chara
#    get 1 character
def get1chara(f):
    chara = f.read(1)
    if chara == "/":
        nextChara = f.read(1)
        if nextChara == "/":
            skipUntilCR(f)              #行末までスキップ
            chara = f.read(1)
        elif nextChara == "*":
            skipUntilSS(f)              #"*/"までスキップ
            chara = f.read(1)
        else:
            f.seek(-1,1)                #1文字戻す
    return chara

def skipUntilCR(f):
    chara = f.read(1)
    while (chara != "\n") and (chara != ""):
        chara = f.read(1)
    f.seek(-1,1)                        #1文字戻す

def skipUntilSS(f):
    chara = f.read(1)
    nextChara = f.read(1)
    while not((chara == "*") and (nextChara == "/")) and (chara != ""):
        chara = nextChara
        nextChara = f.read(1)

#
#  getPatchNameFromBoundary
#    boundaryがあるDirから取得
def getPatchNameFromBoundary(wdir):
    fileName = wdir + "/boundary"
    ans = PatchNameFronBoundary_sub(fileName)
    return ans

#
#  PatchNameFromBoundary
#    polyMeshがあるDirから取得
#    戻り値：[[name, type, nFaces, startFace], ...]
def PatchNameFromBoundary(polyMeshDir):
    fileName = polyMeshDir + "/polyMesh/boundary"
    ans = PatchNameFronBoundary_sub(fileName)
    return ans

def PatchNameFronBoundary_sub(fileName):
    ans = []
    try:
        f = open(fileName)
        contents = f.read()
        f.close
    except:
        print "error: ", polyMeshDir, "は、polyMeshのDirでは、ありません。"
        ans = []
        return
    ans = []
    p=skipFoamFileMem(contents, 0)
    [values, p] = getSmallPairMem(contents, p)
    [[keyword, value], p] = getNextKeywordAndContentsMem(values, 0)
    while keyword != "":
        name = ["", "", "", ""]
        name[0] = keyword
        lines = value.split("\n")
        for line in lines:
            items = line.split()
            if len(items)>1:
                if items[0] == "type":
                    name[1] = items[1]
                elif items[0] == "nFaces":
                    name[2] = items[1]
                elif items[0] == "startFace":
                    name[3] = items[1]
        ans.append(name)
        [[keyword, value], p] = getNextKeywordAndContentsMem(values, p)
    return ans

#
#  cellZoneNames
#    cellZonesからcellZoneNameを取得
def cellZoneNames():
    wdir = os.getcwd() + "/constant/polyMesh"
    Zones = cellZones(wdir)
    zoneNames =[]
    if len(Zones) == 0:
        return zoneNames
    else:
        i=0
        while i < len(Zones):
            zoneNames.append(Zones[i][0])
            i=i+1
        zoneNames.sort()
        return zoneNames


#
#  cellZones
#    cellZonesの内容を取得
def cellZones(wdir):
    fileName = wdir + "/cellZones"
    ans = []
    try:
        f = open(fileName)
        contents = f.readlines()
        f.close
    except:
        print "error: ", wdir, "内に「cellZones」がありません。OpenFOAMのCaseDirかどうか確認してください!!"
        ans = []
        return ans
    contents = deleteComments(contents)
    ans = getCellZonesContents(contents)
    return ans

#
#  faceZones
#    faceZoneの内容を取得
def faceZones(wdir):
    fileName = wdir + "/faceZones"
    ans = []
    try:
        f = open(fileName)
        contents = f.readlines()
        f.close
    except:
        print "error: ", wdir, "は、OpenFOAMのCaseDirではありません!!"
        ans = []
        return
    contents = deleteComments(contents)
    ans = getCellZonesContents(contents)
    return ans


def deleteComments(contents):
    ans = []
    n = 0; c = 0;ansL = ""
    while n < len(contents):
        if len(contents[n]) == 1:
            c = 0; n = n + 1
        else:
            char = contents[n][c:c+2]
            if char[0] == '"':
                # skip next '"'
                ansL = ansL + char[0]
                c = c + 1
                while contents[n][c] != '"' and c < len(contents[n]):
                    ansL = ansL + contents[n][c]
                    c = c + 1
                ansL = ansL + contents[n][c]
                c = c + 1
            elif char == "/*":
                # skip next "*/"
                while contents[n][c:c+2] != "*/" and n < len(contents):
                    c = c + 1
                    if c >= (len(contents[n]) -1):
                        c = 0; n = n + 1
                c = c + 2
            elif char == "//":
                # skip EOL
                c = 0; n = n +1
            else:
                ansL = ansL + contents[n][c]
                c = c + 1
        if n < len(contents):
            if c >= len(contents[n]) -1:
                c = 0; n = n + 1
                ans.append(ansL); ansL = ""
    comp = []
    # 空文を削除
    for cont in ans:
        if cont != "":
            comp.append(cont)
    # 空白を詰める
    n=0
    for cont in comp:
        if len(cont) > 2:
            i=0; line = ""
            while i < len(cont)-1:
                if cont[i:i+2] != "  ":
                    line=line + cont[i]
                i=i+1
            line = line + cont[i]
            if line[0] == " ":
                comp[n] = line[1:]
            else:
                comp[n] = line
        n=n+1
    return comp


def getValue(item, line):
    a=line.split()
    if a[0] == item:
        if a[1][-1] == ";":
            return a[1][:-1]
        else:
            return a[1]
    else:
        return ""

def getCellZonesContents(contents):
    n=0
    FoamFileA = getValueInContents(n, "{", "}", contents)
    n = FoamFileA[0]; FoamFile = FoamFileA[1]
    allValue = []
    while n < len(contents):
        cellZoneValue = getValueInContents(n, "{", "}", contents)
        n = cellZoneValue[0]
        if n < len(contents):
            #cellZoneの内容が1行で記述なのか複数行で記述なのかを確認
            if (len(cellZoneValue[1][-1][-1]) > 1 and
                cellZoneValue[1][-1][-1] != ";"):
                #1行で記述の場合
                cellZoneValue1 = getValueInContents1Line(cellZoneValue[1][-1][-1])
                cellVal = []
                cellVal.append(cellZoneValue[1][1][0])
                for val in cellZoneValue1:
                    cellVal.append(val)
                cellValue = []
                cellValue.append(cellZoneValue[1][0])       #cellZone名
                cellValue.append(cellVal)                   #cellZone内容
                allValue.append(cellValue)                  #データセット
            else:
                #複数行で記述の場合（通常）
                allValue.append(cellZoneValue[1])
    return allValue

#
#  getValueInContents1Line
#   1行内のListの値を取得
def getValueInContents1Line(contents):
    [cellLabel1, P] = getKeywordMem(contents, 0)
    [cellLabel2, P] = getKeywordMem(contents, P)
    cellLabelList = cellLabel1 + " " + cellLabel2
    [value, P] = getSmallPairMem(contents, P)
    valueList = value.split()
    ans = []
    ans.append(cellLabelList)
    ans.append(len(valueList))
    ans.append("(")
    for val in valueList:
        ans.append(val)
    ans.append(")")
    ans.append(";")
    return ans


#
#  getValueInContents
#   複数行に渡るListの値を取得
#    引数：　<n> <stChara> <edChara> [<contents>]
#    戻り値：[<n> [<name> [<val...]]]
#             行数　名前　　値（list）
def getValueInContents(n, stChr, edChr, contents):
    i=n; ans=[]; n=len(contents)
    while i < len(contents):
        if contents[i] == stChr:
            n=i+1
            i=len(contents)
        else:
            i=i+1
    if n < len(contents) and n-2 >= 2:
        valName = contents[n-2]
    else:
        valName = ""
    ans = []
    i = n
    n = len(contents)                          #エラーフラグセット
    while i < len(contents):
        if contents[i] == edChr:
            n=i+1
            i=len(contents)
        else:
            ans.append(contents[i])
        i=i+1
    ansR = []; name = []
    ansR.append(n)
    name.append(valName)
    name.append(ans)
    ansR.append(name)
    return ansR


#
#  test--------------------------------------
import string
if __name__ == "__main__":
    wdir = os.getcwd()
    polyMeshDir0 = "/home/caeuser/CAE/CAE-FOAM/conjugate/conjugateTestModel/conjugateTestCase-DyMesh/subCase/regCase/constant/polyMesh"
    polyMeshDir1 = "/home/caeuser/CAE/CAE-FOAM/conjugate/conjugateTestModel/conjugateTestCase-DyMesh/subCase/regCase/constant/solid2/polyMesh"
    samePatches = getSameLocationPatches(polyMeshDir0, polyMeshDir1)
    print samePatches


