#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   pyTreeFoam.py
#
#    module of treefoam.py
#
#       11/04/18    新規作成
#       12/01/01    replaceBoundaryFieldを修正（ｺﾒﾝﾄ文に対応）
#          01/04    deletePatch（パッチ削除）を追加
#          01/08    changePatchName（パッチ名の変更）を追加
#          01/15    delpatchを修正（最初のpatch削除ができなかった）
#

import os
import glob
import dircache
import string
import getFoamContents


#
#  changePatchName
#    patch名の変更
#
def changePatchName(boundaryDir, dataDir, fields, oldPatchName, newPatchName):
    error = checkPatchName(newPatchName)
    if error[0] != "OK":
        return error
    error = IsOldPatchNameInBoundary(boundaryDir, dataDir, fields, oldPatchName)
    if error[0] != "OK":
        return error
    error = changePatchNameInBoundary(boundaryDir, oldPatchName, newPatchName)
    if error[0] != "OK":
        return error
    error = changePatchNameInBoundaryField(dataDir, fields, oldPatchName, newPatchName)
    return error

#  checkPatchName
def checkPatchName(newPatchName):
    error = ["OK", ""]
    flag = 0
    for chara in newPatchName:
        if getFoamContents.checkChara(chara) == False:
            flag = 1
            break
    if flag == 0:
        if chara == "/":
            flag = 1
        elif chara == "\\":
            flag = 1
        elif chara == "'" or chara == '"':
            flag = 1
    if flag == 1:
        error = ["NG", u"使えない文字「" + chara + u"」が使われています"]
    return error

#  IsOldPatchNameInBoundary
def IsOldPatchNameInBoundary(boundaryDir, dataDir, fields, oldPatchName):
    error = ["OK", ""]
    #boundaryのチェック
    fileName = boundaryDir + "/boundary"
    if glob.glob(fileName) != "":
        f=open(fileName); contents=f.read(); f.close()
        #FoamFileをskip
        p = getFoamContents.skipFoamFileMem(contents, 0)
        #boundary中にoldPatchNameがあるかチェック
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
        while (keyword != "") and (keyword != oldPatchName):
            [keyword, p] = getFoamContents.getKeywordMem(contents, p)
        if keyword == "":
            error = ["NG", u"boundary中に「" + oldPatchName + u"」が存在しません"]
            return error
    else:
        error = ["NG", fileName+u"が存在しません。"]
        return error
    #boundaryFieldのチェック
    for field in fields:
        fileName = dataDir + "/" + field
        f=open(fileName); contents=f.read(); f.close()
        #FoamFileをskip
        p = getFoamContents.skipFoamFileMem(contents, 0)
        #boundaryFieldを取得
        [boundaryContents, p] = getFoamContents.getKeywordContentsMem("boundaryField", contents, p)
        #boundaryField中のoldPatchNameがあるかチェック
        [keyword, p] = getFoamContents.getKeywordMem(boundaryContents, 0)
        while (keyword != "") and (keyword != oldPatchName):
            [keyword, p] = getFoamContents.getKeywordMem(boundaryContents, p)
        if keyword == "":
            error = ["NG", u"boundary中に「" + oldPatchName + u"」が存在しません"]
            return error
    return error

#  changePatchNameInBoundary
def changePatchNameInBoundary(boundaryDir, oldPatchName, newPatchName):
    error = ["OK", ""]
    newContents = []
    fileName = boundaryDir + "/boundary"
    f=open(fileName); contents=f.read(); f.close()
    #FoamFileをskip
    p = getFoamContents.skipFoamFileMem(contents, 0)
    #oldPatchName検索
    [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    oldP = p
    while (keyword != "") and (keyword != oldPatchName):
        oldP = p
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    #boundaryを書き換え
    if (keyword != ""):
        newContents = (contents[:oldP] +
                        "    }\n    " +
                        newPatchName +
                        "\n" +
                        contents[p:])
        f=open(fileName, "w"); f.write(newContents); f.close()
    else:
        error = ["NG", u"「" + oldPatchName + u"」がboundary中に存在しません"]
    return error

#  changePatchNameInBoundaryField
def changePatchNameInBoundaryField(dataDir, fields, oldPatchName, newPatchName):
    error = ["OK", ""]
    for field in fields:
        newContents = []
        fileName = dataDir + "/" + field
        f=open(fileName); contents=f.read(); f.close()
        #pointerをboundaryFieldまで進める
        p = getFoamContents.getKeywordPointerMem("boundaryField", contents, 0)    
        newContents = contents[:p] + "boundaryField\n{\n"
        #boundaryFieldを取得
        [boundaryContents, p] = getFoamContents.getKeywordContentsMem("boundaryField", contents, p)
        [keyword, p] = getFoamContents.getKeywordMem(boundaryContents, 0)
        while (keyword != oldPatchName) and (keyword != ""):
            pbak = p
            [keyword, p] = getFoamContents.getKeywordMem(boundaryContents, p)
            if keyword == oldPatchName:
                newContents += (boundaryContents[:pbak] +
                                "    }\n    " +
                                newPatchName +
                                "\n" +
                                boundaryContents[p:])
        if keyword != "":
            newContents += "}\n\n// ************************************************************************* //"
            f=open(fileName, "w"); f.write(newContents); f.close()
        else:
            error = ["NG", 
                     "「" + delPatch + "」のpatchが「" + field
                     + "」のboundaryField中に存在しません"]
    return error

#
#  deletePatch
#    patchを削除
def deletePatch(boundaryDir, dataDir, fields, delPatch):
    error = delPatchBoundaryFile(boundaryDir, delPatch)
    if error[0] == "OK":
        error = delPatchBoundaryField(dataDir, fields, delPatch)
    return error

#  delPatchBoundaryFile
#    boundary file 中のpatchを削除
def delPatchBoundaryFile(boundaryDir, delPatch):
    error = ["OK", ""]
    fileName = boundaryDir + "/boundary"
    if glob.glob(fileName) != "":
        f=open(fileName); contents=f.read(); f.close()
        #FoamFileをskip
        p = getFoamContents.skipFoamFileMem(contents, 0)
        #patch数を減らす
        [patchN, p] = getFoamContents.getKeywordMem(contents, p)
        stP = p - (len(patchN)+1)           #ポインタをkeywordの前に戻す
        newContents = contents[:stP]        #ポインタまでを書き込み
        n = int(patchN) -1
        newContents += str(n) + "\n("        #patch数を書き込み
        #patchを削除
        [boundaryCont, p] = getFoamContents.getSmallPairMem(contents, p)
        p=0; flag=0
        patchName = ["", ""]
        while (patchName[0] != delPatch) and (p < len(boundaryCont)) and (flag == 0):
            pbak = p
            [patchName, p] = getFoamContents.getNextKeywordAndContentsMem(boundaryCont, p)
            if patchName[0] == delPatch:
                newContents += boundaryCont[:pbak] + boundaryCont[p:]
                flag = 1
        if flag == 1:
            newContents += ")\n\n//************************************************************************* //"
            f=open(fileName, "w"); f.write(newContents); f.close()
        else:
            error = ["NG", "u「" + delPatch + u"」のpatchがboundary中に存在しません"]
    else:
        error = ["NG", fileName + u"が存在しません。"]
    return error

#  delPatchBoundaryField
#    boundaryField中のpatchを削除
def delPatchBoundaryField(dataDir, fields, delPatch):
    error = ["OK", ""]
    for field in fields:
        fileName = dataDir + "/" + field
        f=open(fileName); contents=f.read(); f.close()
        p = getFoamContents.getKeywordPointerMem("boundaryField", contents, 0)    
        newContents = contents[:p]
        [boundaryContents, p] = getFoamContents.getKeywordContentsMem("boundaryField", contents, p)
        newContents += "boundaryField\n{"
        p=0; flag = 0
        patchName = ["", ""]
        while (patchName[0] != delPatch) and (p < len(boundaryContents)) and (flag == 0):
            pbak = p
            [patchName, p] = getFoamContents.getNextKeywordAndContentsMem(boundaryContents, p)
            if patchName[0] == delPatch:
                newContents += boundaryContents[:pbak] + boundaryContents[p:]
                flag = 1
        if flag == 1:
            newContents += "}\n\n// ************************************************************************* //"
            f=open(fileName, "w"); f.write(newContents); f.close()
        else:
            error = ["NG", 
                     u"「" + delPatch + u"」のpatchが「" + field
                     + u"」のboundaryField中に存在しません"]
    return error

#
#  setBoundaryData
#    boundary（patchの定義）を保存する
def setBoundaryData(boundaryDir, patches, defineData):
    #print defineData
    error = ["OK",""]
    fileName = boundaryDir + "/boundary"
    if glob.glob(fileName) != "":
        f=open(fileName); contents=f.read(); f.close()
        newDefine = makeDefineData(patches, defineData, contents)
        #print newDefine    
        repDefine = replaceDefineData(patches, newDefine, contents)
        #print repDefine
        if repDefine != "":
            f=open(fileName,"w"); f.write(repDefine); f.close()
        else:
            error = ["NG", fileName+u"のデータが取得できません。"]
    else:
        error = ["NG", fileName+u"が存在しません。"]
    return error


def replaceDefineData(patches, newDefine, contents):
    #boundaryから（）内データを取得
    p=0
    [dummy, p] = getFoamContents.getKeywordContentsMem("FoamFile", contents, p)
    [defineFile, p] = getFoamContents.getSmallPairMem(contents, p)
    if defineFile != "":
        #ポインタを（）内データの頭と後ろに設定しておく
        p = p - len(defineFile)
        pe = p + len(defineFile) - 1
        #patch毎に内容を入替え
        for patch in patches:
            defineFile = replaceKeywordData(patch, newDefine, defineFile)
        repData = contents[:p] + newDefine + contents[pe:]
    else:
        repData = ""
    return repData


def replaceKeywordData(patch, newDefine, defineFile):
    patch = patch.split()[0]
    [orgCon, p] = getFoamContents.getKeywordContentsMem(patch, defineFile, 0)
    if orgCon != "":
        [newCon, n] = getFoamContents.getKeywordContentsMem(patch, newDefine, 0)
        p = p - len(orgCon)
        repData = defineFile[:p] + newCon + defineFile[p+len(orgCon)-1:]
    else:
        repData = defineFile
    return repData


def makeDefineData(patches, defineData, contents):
    #patchデータを整形
    define = makeBoundaryField(patches, defineData)
    #各patch毎にdefineデータを作成
    newDefine = ""
    for patchSP in patches:
        patch = patchSP.split()[0]
        [definePatch, p] = getFoamContents.getKeywordContentsMem(
                        patch, define, 0)
        [defineFile, p] = getFoamContents.getKeywordContentsMem(
                        patch, contents, 0)
        definePatch = reformTabContents(definePatch)
        defineFile  = reformTabContents(defineFile)
        #nFaces, startFaceは、隠す。
        addLine = ""
        if definePatch != "" and defineFile != "":
            lines = defineFile.split("\n")
            for line in lines:
                if string.find(line, "nFaces")>=0:
                    addLine += line+"\n"
                if string.find(line, "startFace")>=0:
                    addLine += line+"\n"
            newDefine += patchSP + "\n    {\n"
            newDefine += definePatch + addLine + "    }\n"
    return newDefine


#  reformTabContents
#    行頭に空白、行中のtab追加
#    tab:8,20で設定
def reformTabContents(value):
    tab1=8; tab2=20
    #各行に分割
    lines = value.split("\n")
    #各行のnul行を削除
    words = []
    for line in lines:
        ll = line.split()
        if len(ll) > 0:
            if ll[0] != "":
                words.append(line)
    #各行にtab追加
    i=0
    line=""
    while i<len(words):
        #行を単語に分割
        wds = words[i].split()
        if len(wds) > 1:
            #2単語以上の場合、行頭に空白(tab1個)追加
            wds[0] = tab1*" " + wds[0]
            if len(wds[0])<tab2-1:
                #第1単語がtab2より短い場合、tab2位置まで空白追加
                wds[0] += (tab2 - len(wds[0]))*" "
            else:
                #長い場合は、空白1ヶを追加
                wds[0] += " "
            #1行を作成
            line += wds[0] + string.join(wds[1:], " ") + "\n"
        elif len(wds) == 1:
            #行内に1単語のみの場合
            line += tab1*" " + wds[0] + "\n"
        i+=1
    return line

#
#  setFieldsPatchesName
#    boundaryConditionをfileにセット
#    「getFieldsPatchesName」と反対の処理
#     文字が多すぎる場合、
def setFieldsPatchesName(dataDir, fields, patches, patchData, intData):
    i=0
    mess = ""
    while i<len(fields):
        #fieldをオープン
        field = fields[i]
        fileName = dataDir + "/" + field
        f=open(fileName); contents=f.read(); f.close()
        #internalFieldを作成、置き換え
        contents = makeReplaceInternalField(fields, intData[i], contents)
        #patchDataを元に戻す（"..."がある場合）
        patchDataH = remakeBoundaryField(contents, patches, patchData[i])
        #patchDataから「boundaryField」を作成
        newBoundary = makeBoundaryField(patches, patchData[i])
        #「boundaryField」を置き換える
        newContents = replaceBoundaryField(newBoundary, contents)
        if newContents != "":
            f=open(fileName,"w"); f.write(newContents); f.close()
        else:
            print "error:",field,"が置き換えできませんでした。"
            mess += field + ","
        i+=1
    if mess == "":
        error = ["OK", ""]
    else:
        error = ["NG", mess[:-1]+u"が書き込めません。"]
    return error


def remakeBoundaryField(contents, patches, patchData):
    i=0
    while i<len(patchData):
        if patchData[i][-4:-1]=="..." or patchData[i][-3:]=="...":
            patchData[i] = replacePatchData(contents, patches[i], patchData[i])
        i+=1
    return patchData


def replacePatchData(contents, patch, data):
    p=getFoamContents.skipFoamFileMem(contents, 0)
    [dummy, p] = getFoamContents.getKeywordContentsMem(
                        "boundaryField", contents, p)
    #patchにスペースを追加してあるので、スペースを詰める
    [patchSp, p] = getFoamContents.getKeywordMem(patch, 0)
    #元のデータからpatchデータを取得
    [patchCont, p] = getFoamContents.getKeywordContentsMem(patchSp, dummy, 0)
    #最後のスペースを詰める
    i=len(patchCont)-1
    while i>0 and patchCont[i] ==" ":
        i-=1
    patchCont = patchCont[:i+1]
    return patchCont
    

def makeReplaceInternalField(fields, intData, contents):
    p=getFoamContents.skipFoamFileMem(contents, 0)
    [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    while keyword != "internalField" and keyword != "":
        [keyword, p] = getFoamContents.getKeywordMem(contents, p)
    if keyword != "":
        p = p - (len("internalField")+1)
        line = contents[p:p+100]
        n = string.find(line, " uniform ")
        if n >= 0:
            m = string.find(line, "\n")
            contents = contents[:p] + intData + contents[p+m+1:]
        else:
            n = string.find(intData, " uniform ")
            if n>= 0:
                [dummy, m] = getFoamContents.getKeywordContentsMem(
                        "boundaryField", contents, p)
                m = m - len(dummy)
                contents = contents[:p] + intData + "\nboundaryField\n{\n" + contents[m:]
    return contents 


def makeBoundaryField(patches, patchData):
    boundary = ""
    i=0
    lenPD=len(patchData)
    while i<lenPD:
        #patchDataが空文の場合、作成しない。
        if (patchData[i] != ""):
            patch = patches[i]
            boundary += patch + "\n    {\n"
            boundary += patchData[i]
            boundary += "    }\n"
        i+=1
    return boundary

def replaceBoundaryField(newBoundary, strings):
    #boundaryのポインタを取得
    p=0
    pp=getFoamContents.getKeywordPointerMem(
                        "boundaryField", strings, p)
    #boundaryの内容を取得
    [boundary, p] = getFoamContents.getKeywordContentsMem(
                        "boundaryField", strings, pp)
    #ポインタをboundaryの頭に戻す
    if boundary != "":
        p = pp
        strings = (strings[:p]
                    + "boundaryField\n{\n"
                    + newBoundary
                    + "}\n\n// ************************************************************************* //")
    else:
        strings = ""
    return strings

#
#  getFieldsPatchesName
#    各Fieldのpatch名とboundaryConditionを取得
#    戻り値：[[fields], [patchs], [boundaryConditions]]
def getFieldsPatchesName(editDir, selectedFields):
    wdir = editDir + "/" + selectedFields[0][0]
    files = selectedFields[0][1:]
    [files, patches, patchData] = getFieldsPatchesName_sub(wdir, files)
    return [files, patches, patchData]

#
#  getAllFieldsPatchesName
#    各Fieldのpatch名とboundaryConditionを取得
#    戻り値：[[fields], [patchs], [boundaryConditions]]
def getAllFieldsPatchesName(wdir):
    [folders, files] = getFoldersFiles(wdir)
    [files, patches, patchData] = getFieldsPatchesName_sub(wdir, files)
    return [files, patches, patchData]

#  getFieldsPatcesName_sub
def getFieldsPatchesName_sub(wdir, files):
    fieldsPatch = []
    patchData = []
    for name in files:
        patchesCont = []
        fileName = wdir + "/" + name
        f=open(fileName); contents=f.read(); f.close()
        p=0
        [boundary, p] = getFoamContents.getKeywordContentsMem(
                            "boundaryField", contents, p)
        [patchCont, p] = getFoamContents.getNextKeywordAndContentsMem(
                            boundary, 0)
        while patchCont[0] != "":
            patchesCont.append(patchCont)
            [patchCont,p]=getFoamContents.getNextKeywordAndContentsMem(
                                boundary, p)
        fieldsPatch.append(patchesCont)
    patches = []
    for patchC in fieldsPatch[0]:
        patches.append(patchC[0])
    for fieldPatch in fieldsPatch:
        col=[]
        for patch in patches:
            col.append("")
        for fpatch in fieldPatch:
            flag = 0; i=0
            for patch in patches:
                if fpatch[0] == patch:
                    col[i]=compressSPCR(fpatch[1])
                    flag = 1
                    break
                i+=1
            if flag == 0:
                patches.append(fpatch[0])
                col.append(compressSPCR(fpatch[1]))
        patchData.append(col)
    return [files, patches, patchData]

def compressSPCR(contents):
    lines = contents.split("\n")
    ans = []
    for line in lines:
        if line != "":
            wordList = line.split()
            ans.append(string.join(wordList, " "))
    ansR = string.join(ans, "\n")
    if ansR[-1] == "\n":
        ansR = string.join(ans, "\n")[:-1]
    return ansR

#
#  remakeFolderConjugate
#    folderの位置をconjugate用に配置する
def remakeFolderConjugate(fluidZone, solidZone):
    caseDir = os.getcwd()
    regDir = caseDir + "/regCase"
    wdir = caseDir.split("/")[1:-1]
    timeFolders = getTimeFolders(caseDir)
    timeFolder = timeFolders[0]

    #流体側のcellZoneを処理
    fluidName = fluidZone[0]
    print "流体側のzellZone「", fluidName, "」を整形中..."
    #  meshを移動
    sourceDir = regDir + "/" + timeFolder + "/" + fluidName + "/polyMesh"
    toDir = regDir + "/constant/polyMesh"
    moveFile(sourceDir, toDir)
    #　fieldを移動
    sourceDir = regDir + "/" + timeFolder + "/" + fluidName
    toDir = regDir + "/" + timeFolder
    moveFile(sourceDir, toDir)
    #  不要なfolderを削除
    os.system("rm -r " + regDir + "/" + timeFolder + "/" + fluidName + "> /dev/null")
    os.system("rm -r " + regDir + "/constant/" + fluidName + " > /dev/null")
    os.system("rm -r " + regDir + "/system/" + fluidName + " > /dev/null")

    #固体側のcellZoneを処理
    i=1
    for solidName in solidZone:
        print "固体側「", solidName, "」を整形中..."
        #folder作成
        toDir = caseDir + "/" + solidName
        os.system("mkdir " + toDir)
        #constantをコピー
        sourceDir = regDir + "/"+ timeFolder + "/" + solidName + "/polyMesh"
        toDir = caseDir + "/" + solidName + "/constant"
        os.system("mkdir " + toDir)
        toDir += "/polyMesh"
        os.system("mkdir " + toDir)
        moveFile(sourceDir, toDir)
        os.system("rm -r " + sourceDir + " > /dev/null")
        os.system("rm -r " + regDir + "/constant/" + solidName)
        source = caseDir + "/" + solidName + "/constant"
        toLink = "./regCase/constant/solid" + str(i)
        os.system("ln -s " + source + " " + toLink)
        #timeFolderをコピー
        sourceDir = regDir + "/" + timeFolder + "/" + solidName
        toDir = caseDir + "/" + solidName + "/" + timeFolder
        os.system("mkdir " + toDir)
        moveFile(sourceDir, toDir)
        os.system("rm -r " + sourceDir)
        source = caseDir + "/" + solidName + "/" + timeFolder
        toLink = "./regCase/" + timeFolder + "/solid" + str(i)
        os.system("ln -s  " + source + " " + toLink)
        #systemをコピー
        sourceDir = regDir + "/system"
        toDir = caseDir + "/" + solidName + "/system"
        os.system("mkdir " + toDir)
        os.system("cp "+ sourceDir+"/controlDict " + toDir + " > /dev/null")
        os.system("cp "+ sourceDir+"/fv* " + toDir + " > /dev/null")
        os.system("rm -r " + regDir + "/system/" + solidName)
        source = caseDir + "/" + solidName + "/system"
        toLink = "./regCase/system/solid" + str(i)
        os.system("ln -s " + source + " " + toLink)

        i=i+1

    print "メッシュをconjugate用に分割しました。"


def moveFile(sourceDir, toDir):
    comm = "cp " + sourceDir + "/* " + toDir + "> /dev/null"
    os.system(comm)
    comm = "rm -r " + sourceDir + "/* > /dev/null"
    os.system(comm)

#
#  getTimeFolders
#    caseDir内のtimeFolderを取得
def getTimeFolders(caseDir):
    dirs = glob.glob(caseDir + "/*")
    folders = []
    for folder in dirs:
        folders.append(folder.split("/")[-1])
    time=[]
    for name in folders:
        try:
            a=float(name)
            time.append([a, name])
        except:
            a=0
    time.sort()
    timeFolders=[]
    for name in time:
        timeFolders.append(name[1])
    return timeFolders

#
#  getFoldersFiles
#    dir内のfolder名とfile名を返す（名前のみ返す。pathではない）
#    戻り値：[[folders],[files]]
def getFoldersFiles(wdir):      #wdirは、絶対path
    dircache.reset()
    a = dircache.listdir(wdir)
    dircache.annotate(wdir, a)
    files = []; folders = []
    for name in a:
        if name[-1] == "/":
            folders.append(name[:-1])
        else:
            if name != "" and name[-1] != "~":
                files.append(name)
    ans = []
    ans.append(folders)
    ans.append(files)
    return ans

#
#  runOFUtility
#    OpenFOAMのutilityを実行する
def runOFUtility(comm):
    bashrc = getBashrcFoam()
    cont = ""
    cont += ". " + bashrc + "; "
    cont += comm
    os.system(cont)
    return

#
# getBashrcFoam
#   外からOpenFOAMの環境設定ファイルを読み込む時に使用
#   環境変数$TreeFoamPathから取得する
def getBashrcFoam():
    treeFoamPath = os.environ["TreeFoamPath"]
    if treeFoamPath == "":
        print "環境変数$TreeFoamPathが設定されていないので、隠しファイル~/.treeFoamInstallFolderから、TreeFoamPathを取得します。"
        homeFolder = os.environ["HOME"]
        f = open(homeFolder + "/.treeFoamInstallFolder")
        treeFoamPath = f.readline()
        f.close()
    f = open(treeFoamPath + "/configTreeFoam")
    configContents = f.readlines()
    f.close()
    for line in configContents:
        word = line.split()
        if (len(word) > 0):
            if (word[0] == "bashrcFOAM"):
                if len(word) >1:
                    envOpenFOAMFix = word[1]
                else:
                    envOpenFOAMFix = ""
                break
    return envOpenFOAMFix

