#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  setFoamContents
#      Foam形式のデータを作成する
#
#       10/03/04    新規作成
#

import os
import glob
import re
import getFoamContents

workdir = ""
Pname = []

#
#  setMachinesDefault
#    defaultのmachinesを作成する
#
def setMachinesDefault():
    wdir = os.getcwd()
    cont = makeMachinesDefault()
    fileName = "./machines"
    f=open(fileName, "w")
    f.write(cont)
    f.close()

def makeMachinesDefault():
    cont = """#  subDomain名とcpu数を記述します
#  subDomain名は、「/etc/hosts」に記述されているhost名を記述
#
#入力例：
#localhost cpu=4
#host1 cpu=2
#host2

localhost cpu=4
"""
    return cont


#
#  setDecomposeParDictDefault
#    defaultのdecomposeParDictを作成する
#
def setDecomposeParDictDefault():
    wdir = os.getcwd()
    cont = makeDecomposeParDictDefault()
    fileName = wdir + "/system/decomposeParDict"
    f=open(fileName, "w")
    f.write(cont)
    f.close()

def makeDecomposeParDictDefault():
    cont = makeFoamHeader()
    loc = "system"
    obj = "decomposeParDict"
    foamFile = makeFoamFile("2.0", "ascii", "dictionary", loc, obj )
    cont = cont + foamFile
    a = (
"""
numberOfSubdomains 4;

//- Keep owner and neighbour on same processor for faces in zones:
// preserveFaceZones (heater solid1 solid3);

//- Keep owner and neighbour on same processor for faces in patches:
//  (makes sense only for cyclic patches)
//preservePatches (cyclic_left_right);


method          simple;
// method          hierarchical;
// method          simple;
// method          metis;
// method          manual;

simpleCoeffs
{
    n               ( 2 2 1 );
    delta           0.001;
}

hierarchicalCoeffs
{
    n               ( 1 1 1 );
    delta           0.001;
    order           xyz;
}

metisCoeffs
{
    processorWeights ( 1 1 1 1 );
}

manualCoeffs
{
    dataFile        "";
}

distributed     no;

roots           ( );
//// Is the case distributed
//distributed     yes;
//// Per slave (so nProcs-1 entries) the directory above the case.
//roots           
//(
//    "/tmp"
//    "/tmp"
//);

// ************************************************************************* //
"""
        )
    cont = cont + a
    return cont
    

#
#  setFieldsDictDefault
#    defaultのsetFieldsDictを作成する
#
def setFieldsDictDefault():
    wdir = os.getcwd()
    files = glob.glob(wdir + "/system")
    if len(files) != 0:
        #setFieldsDictの作成
        cont = makeSetFieldsDictDefault()
        fileName = "./system/setFieldsDict"
        f=open(fileName,"w")
        f.write(cont)
        f.close
    else:
        print "systemフォルダがないので、「setFieldsDict」が作れません。"

def makeSetFieldsDictDefault():
    cont = makeFoamHeader()
    location = "system"
    foamFile = makeFoamFile("2.0", "ascii", "dictionary", location, "setFieldsDict" )
    cont = cont + foamFile
    a = (
'''defaultFieldValues
(
    volScalarFieldValue xxxxx 0                 // scalar
    volVectorFieldValue yyyyy (0 0 0)           //vector
);

regions
(
    cellToCell
    {
        set cellZone_name1;                     //cellZoneName
        fieldValues
        (
            volScalarFieldValue xxxxx 10
            volVcalarFieldValue yyyyy (1 1 1)
        );
    }

    cellToCell
    {
        set cellZone_name2;                     //cellZoneName
        fieldValues
        (
            volScalarFieldValue xxxxx 1
            volVcalarFieldValue yyyyy (2 2 2)
        );
    }
);

// ************************************************************************* //
'''
        )
    cont = cont + a
    return cont


#
#  controlDict
#    controlDictの内容を書き換える
#    items: [[item, value], [item, value], ...]
def controlDict(wdir, items):
    fileName = wdir + "/controlDict"
    f = open(fileName)
    contents = f.readlines()
    f.close()
    for item in items:
        n=0
        while n < len(contents):
            try:
                i = contents[n].index(item[0])
                if i == 0:
                    line = exchangeValue(item[1], contents[n])
                    contents[n] = line
                n=n+1
            except:
                n=n+1
    f = open(fileName, "w")
    for line in contents:
        f.write(line)
    f.close()
    
def exchangeValue(value, cont):
    a = cont.split()
    i = cont.index(a[1])
    line = cont[:i] + value + ";" + cont[i+len(a[1]):]
    return line

#
#  faceZones
#    faceZonesの内容を書き込む（flipMapは{1}が前提でfaceZoneを書き込む）
#   引数: <wdir> [<faceZoneName>, [value, value, ...], <faceZoneName>, [value, ...]]
#
def faceZones(wdir, contents):
    fileName = wdir + "/faceZones"
    writeCont = []
    header = makeFoamHeader()
    location = "constant/polyMesh"
    FoamFile = makeFoamFile("2.0", "ascii", "regIOobject", location, "faceZones")
    footer = makeFooter()
    writeCont.append(header)
    writeCont.append(FoamFile)
    middle = makeFaceZonesMiddle()
    writeCont.append(str(len(contents))+"\n(\n")
    for cont in contents:
        name = cont[0]
        writeCont.append(name + "\n{\n")
        writeCont.append(middle)
        writeCont.append(str(len(cont[1])) + "\n(\n")
        i=0
        while i < len(cont[1]):
            writeCont.append(cont[1][i] + "\n")
            i=i+1
        writeCont.append(")\n;\n")
        writeCont.append("flipMap         List<bool> " + str(len(cont[1])) + "{1};\n}\n\n")
    writeCont.append(")\n")
    writeCont.append(footer)
    f = open(fileName, "w")
    for line in writeCont:
        f.write(line)
    f.close

#
#  cellZones
#    cellZonesの内容を書き込む
#   引数: <wdir> [<cellZoneName>, [value, value, ...], <cellZoneName>, [value, ...]]
#
def cellZones(wdir, contents):
    fileName = wdir + "/cellZones"
    writeCont = []
    header = makeFoamHeader()
    location = "constant/polyMesh"
    FoamFile = makeFoamFile("2.0", "ascii", "regIOobject", location, "cellZones")
    footer = makeFooter()
    writeCont.append(header)
    writeCont.append(FoamFile)
    middle = makeCellZonesMiddle()
    writeCont.append(str(len(contents))+"\n(\n")
    for cont in contents:
        name = cont[0]
        writeCont.append(name + "\n{\n")
        writeCont.append(middle)
        writeCont.append(str(len(cont[1])) + "\n(\n")
        i=0
        while i < len(cont[1]):
            writeCont.append(cont[1][i] + "\n")
            i=i+1
        writeCont.append(")\n;\n}\n\n")
    writeCont.append(")\n")
    writeCont.append(footer)
    f = open(fileName, "w")
    for line in writeCont:
        f.write(line)
    f.close

#
#  setsCellZone
#    setsフォルダ内にcellZoneファイルを書き込む
#   引数: <wdir> [<cellZoneName>, [value, value, ...]]
#
def setsCellZone(wdir, contents):
    fileName = wdir + "/" + contents[0]
    writeCont = []
    header = makeFoamHeader()
    location = "constant/polyMesh/sets"
    FoamFile = makeFoamFile("2.0", "ascii", "cellSet", location, contents[0])
    footer = makeFooter()
    writeCont.append(header)
    writeCont.append(FoamFile)
    writeCont.append(str(len(contents[1]))+"\n(\n")
    i=0
    while i < len(contents[1]):
        writeCont.append(contents[1][i] + "\n")
        i=i+1
    writeCont.append(")\n")
    writeCont.append(footer)
    f = open(fileName, "w")
    for line in writeCont:
        f.write(line)
    f.close

#
#  setsFaceZone
#    setsフォルダ内にfaceZoneファイルを書き込む
#   引数: <wdir> [<faceZoneName>, [value, value, ...]]
#
def setsfaceZone(wdir, contents):
    fileName = wdir + "/" + contents[0]
    writeCont = []
    header = makeFoamHeader()
    location = "constant/polyMesh/sets"
    FoamFile = makeFoamFile("2.0", "ascii", "faceSet", location, contents[0])
    footer = makeFooter()
    writeCont.append(header)
    writeCont.append(FoamFile)
    writeCont.append(str(len(contents[1]))+"\n(\n")
    i=0
    while i < len(contents[1]):
        writeCont.append(contents[1][i] + "\n")
        i=i+1
    writeCont.append(")\n")
    writeCont.append(footer)
    f = open(fileName, "w")
    for line in writeCont:
        f.write(line)
    f.close
    
#
# makeFoamHeader
#
def makeFoamHeader():
    a=(
"""/*--------------------------------*- C++ -*----------------------------------*\\
| =========                 |                                                 |
| \\\\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |
|  \\\\    /   O peration     | Version:  1.6.x                                 |
|   \\\\  /    A nd           | Web:      www.OpenFOAM.org                      |
|    \\\\/     M anipulation  |                                                 |
\*---------------------------------------------------------------------------*/
"""
      )
    return a

#
#  makeFoamFile
#
def makeFoamFile(version, format, className, location, objectName):
    a=  "FoamFile\n"
    a=a+"{\n"
    a=a+"    version     " + version + ";\n"
    a=a+"    format      " + format + ";\n"
    a=a+"    class       " + className + ";\n"
    a=a+'    location    "' + location + '";\n'
    a=a+"    object      " + objectName + ";\n"
    a=a+"}\n"
    a=a+"// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n"
    a=a+"\n"
    return a

# makeCellZonesMiddle
def makeCellZonesMiddle():
    a=  "    type cellZone;\n"
    a=a+"cellLabels      List<label>\n"
    return a

# makeFaceZonesMiddle
def makeFaceZonesMiddle():
    a=  "    type faceZone;\n"
    a=a+"faceLabels      List<label>\n"
    return a

# makeFooter
def makeFooter():
    a="\n//************************************************************************* //\n"
    return a



#
#  clearBoundaryFields
#
#   clearBoundaryFields(fieldDir, polyMeshDir, Flist)
#     boundaryFieldのクリア
#       filedDir:       各fieldのDir
#       polyMeshDir:    polyMeshのDir
#       Flist:          クリアするFieldのみのlist [field1, field2, ...]
def clearBoundaryFields(fieldDir, polyMeshDir, Flist):
    # patch name の取得
    PnameList = getFoamContents.PatchNameFromBoundary(polyMeshDir)
    if PnameList != None:
        Pname = []
        for name in PnameList:
             Pname.append(name[0])
        # boundary field の作成
        Bcont = "type    zeroGradient;"
        cont = createBoundary(Pname, Bcont)
        # field の作成
        remakeBoundaryFields(fieldDir, Flist, cont)

#
#  clearInternalFields
#
def clearInternalFields(fieldDir, polyMeshDir, Flist):
    contAns = []
    if len(Flist) != 0:
        print "internal field をクリアします..."
        for name in Flist:
            print "    " + name + "..."
            fileName = fieldDir + "/" + name
            f = open(fileName)
            cont = f.readlines()
            f.close
            contAns = clearInternal(cont)
            if len(contAns) != 0:
                f = open(fileName,"w")
                for line in contAns:
                    f.write(line)
                f.close
        print "internal field をクリアしました。"

def clearInternal(cont):
    if len(cont) != 0:
        n=0
        while n < len(cont):
            if re.search("internalField", cont[n]) != None:
                value = cont[n].split()
                if value[1] == "nonuniform":
                    cont = deleteListData(n, cont)
                elif value[1] == "uniform":
                    n = len(cont)
                else:
                    print "error: 形式がありません。"
                    n = len(cont)
                    cont = []
            n=n+1
    return cont

def deleteListData(n, cont):
    valType = internalFieldValueType(cont[n])
    if valType == "scalar":
        value = "0"
    elif valType == "vector":
        value = "(0 0 0)"
    elif valType == "tensor":
        value = "(0 0 0 0 0 0 0 0 0)"
    else:
        # エラーの場合は、何もせずそのまま値を返す。
        value = ""
        print "error: 変数のtypeが存在しません。"
        return cont

    # Field データを削除する。
    cont[n] = "internalField   uniform " + value + ";\n"
    contAns = []
    line=0
    while line <= n:
        contAns.append(cont[line])
        line = line +1
    # 次の")"までスキップ
    matchLine = skipUntilStringEnd(n, "(", ")", cont)
    if matchLine == 0:
        contAns = []
        return contAns
    n = matchLine + 1
    # ";"までスキップ
    matchLine = skipUntilString(n, ";", cont)
    if matchLine == 0:
        contAns = []
        return contAns
    # internalField以外のデータを取り込む
    n = matchLine + 1
    while n < len(cont):
        contAns.append(cont[n])
        n=n+1
    return contAns

def internalFieldValueType(line):
    val = line.split()
    a = val[2].split("<")
    valType = a[1].split(">")
    return valType[0]

def skipUntilStringEnd(st, stChara, edChara, cont):
    n=st; matchLine = 0
    while n < len(cont):
        c = checkCharaInLine(stChara, cont[n])
        if c != None:
            n=n+1
        else:
            c = checkCharaInLine(edChara, cont[n])
            if c != None:
                matchLine = n
                n = len(cont)
            else:
                n=n+1
    return matchLine

def checkCharaInLine(chara, line):
    try:
        c = line.index(chara)
    except:
        c=None
    return c

def skipUntilString(st, string, cont):
    n=st; matchLine = 0
    while n < len(cont):
        try:
            c = cont[n].index(string)
            matchLine = n
            n = len(cont) 
        except:
            n=n+1
    return matchLine        

def createBoundary(Pname,Bcont):
    cont = ""
    for name in Pname:
      cont = cont + "    " + name + "\n"
      cont = cont + "    {\n"
      cont = cont + "        " + Bcont + "\n"
      cont = cont + "    }\n"
    cont = "{\n" + cont + "}\n"
    return cont

def remakeBoundaryFields(fieldDir, Flist, cont):
    fieldNames = []
    i=0
    while i < len(Flist):
        fieldNames.append(Flist[i])
        i=i+1
    remakeTergetBoundaryFields(fieldDir, fieldNames, cont)

def remakeTergetBoundaryFields(fieldDir, fieldNames, cont):
    print fieldDir, "内の各Fieldをクリアします。"
    for name in fieldNames:
        print "    " + name + "..."
        f = open(fieldDir + "/" + name)
        fileCont = []
        for line in f.readlines():
            fileCont.append(line)
        i=0; heder = ""
        while (i < len(fileCont)):
            line = fileCont[i]
            if line[:8] == "boundary":
                heder = heder + line
                i = len(fileCont) + 1
            else:
                heder = heder + line
                i = i + 1        
        if i != len(fileCont) + 1:
            print "      baoundaryFieldがありません。追加しました。"
            heder = heder + "boundaryField\n"
        contAll = heder + cont + "// ************************************************************************* //\n"
        f.close
        fileName = fieldDir + "/" + name
        f = open(fileName,"w")
        f.write(contAll)
        f.close
    print "boundaryFieldをクリアしました。"


