#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   splitMeshCellZones.py
#       メッシュをcellZone毎に分割する。
#       cellZone名とcellNoは、cellZonesファイル内容から取得する。
#       領域分割後、分割したファイルは、初期時間フォルダに移動する。オリジナルの初期時間
#       フォルダは、「before-region.<初期時間フォルダ>」でバックアップされる。
#
#       使い方
#       splitMeshCellZones
#
#       10/03/26    新規作成
#       11/02/25    -cellZonesOnlyに変更（domainが作成される為）
#       11/04/15    OpenFOAMの環境設定ファイルをpyTreeFoam側から読み込む
#

import os
import glob
import shutil
import pyOFLauncher
import string
import setFoamContents
import pyTreeFoam


#
#  splitMeshRegions
#
def splitMeshRegions():
    bashrc = pyTreeFoam.getBashrcFoam()
    command = ". " + bashrc + "; splitMeshRegions -cellZonesOnly"
    os.system(command)

#
#  makeRegNameIntoConstant
#    regionNameのフォルダを作成
def makeRegNameIntoConstant(regCaseDir, timeFolder):
    regNameDir = glob.glob(regCaseDir + "/" + timeFolder + "/*")
    for name in regNameDir:
        folder = name.split("/")[-1]
        os.mkdir(regCaseDir + "/constant/" + folder)
    print "\nconstantフォルダ内にregionNameのフォルダを作成しました。"

#
#  remakeBoundaryFile(regCase)
#    boundary file のpatchを修正
def remakeBoundaryFile(regCaseDir, timeFolder):
    print "\nbounadaryファイルのpatchTypeを修正します。"
    regNameDir = glob.glob(regCaseDir + "/" + timeFolder + "/*")
    regNames = []
    for name in regNameDir:
        regNames.append(name.split("/")[-1])
    boundaryFiles = []
    for nameDir in regNameDir:
        boundaryFiles.append(nameDir + "/polyMesh/boundary")
    for boundary in boundaryFiles:
        f=open(boundary)
        contents = []
        for line in f.readlines():
            contents.append(line)
        f.close()
        for beforeName in regNames:
            for afterName in regNames:
                patch = beforeName + "_to_" + afterName
                i=0
                while i<len(contents):
                    if (string.find(contents[i], patch) >= 0):
                        flag = 0
                        while i<len(contents) and flag < 2:
                            n = string.find(contents[i], "region0")
                            m = string.find(contents[i], "none")
                            if n>=0:
                                contents[i] = contents[i][:n] + afterName + contents[i][n+7:]
                                flag = flag + 1
                            if m>=0:
                                rePatch = afterName + "_to_" + beforeName
                                contents[i] = contents[i][:m] + rePatch + contents[i][m+4:]
                                flag = flag + 1
                            i=i+1
                    i=i+1
        regName = boundary.split("/")[-3]
        print "    " + regName + "の boundary を修正しました。"
        f=open(boundary,"w")
        for line in contents:
            f.write(line)
        f.close()

#
#  remakeBoundaryFileConjugate
#    boundary file をconjugate用に修正
def remakeBoundaryFileConjugate(regCaseDir, timeFolder, fluidReg):
    print "\nbounadaryファイルをconjugate用に修正します。"
    regNameDir = glob.glob(regCaseDir + "/" + timeFolder + "/*")
    regNames = []
    for name in regNameDir:
        regNames.append(name.split("/")[-1])
    boundaryFiles = []
    for nameDir in regNameDir:
        boundaryFiles.append(nameDir + "/polyMesh/boundary")
    #solidRegionを取得
    solidRegs = []
    for solidReg in regNames:
        if solidReg != fluidReg:
            solidRegs.append(solidReg)
    solidRegs.sort()
    print "solidRegs=", solidRegs
    #各region毎に修正
    for boundary in boundaryFiles:
        f=open(boundary)
        contents = []
        for line in f.readlines():
            contents.append(line)
        f.close()
        #項目入替え
        i=0
        while i<len(contents):
            #type変更
            contents[i] = checkCharas(contents[i], 
                            "directMappedWall", "regionCouple")
            #region項目変更
            contents[i] = checkCharas(contents[i],
                            "sampleRegion", "shadowRegion")
            #fluidRegion名変更(region0)
            contents[i] = checkWCharas(contents[i],
                            "shadowRegion", fluidReg, "region0")
            #solidRegion名変更(solid1, solid2, ...)
            contents[i] = checkWsolidCharas(contents[i],
                            "shadowRegion", solidRegs)
            #patch項目変更
            contents[i] = checkCharas(contents[i],
                            "samplePatch", "shadowPatch")
            #offSet項目変更
            contents[i] = checkOffsetLine(contents[i])
            i += 1        
        #"sampleMode"の行削除
        i=0
        repContents = []
        while i<len(contents):
            n = string.find(contents[i], "sampleMode")
            if n<0:
                repContents.append(contents[i])
            i += 1
        #終了処理(repContentsを保存)
        regName = boundary.split("/")[-3]
        print "    " + regName + "の boundary を修正しました。"
        f=open(boundary,"w")
        for line in repContents:
            f.write(line)
        f.close()

def checkCharas(line, orgName, repName):
    n = string.find(line, orgName)
    if n>=0:
        line = line[:n] + repName + line[n+len(orgName):]
    return line

def checkWsolidCharas(line, orgName, repNames):
    if string.find(line, orgName) >= 0:
        i=0
        while i<len(repNames):
            n = string.find(line, repNames[i])
            if n>=0:
                line = line[:n] + "solid" + str(i+1) + line[n+len(repNames[i]):]
                break
            i += 1
    return line

def checkWCharas(line, orgName1, orgName2, repName):
    if string.find(line, orgName1) >= 0:
        n = string.find(line, orgName2)
        if n>=0:
            line = line[:n] + repName + line[n+len(orgName2):]
    return line

def checkOffsetLine(line):
    n = string.find(line, "offset")
    if n>=0:
        line = line[:n] + "attached        on;\n"
    return line

#
#  remakeBoundaryRegionCoupling
#    Region同士の境界面のboundaryをRegionCoupling、remoteFieldに設定する
def remakeBoundaryRegionCoupling(regCase, timeFolder, fields):
    wdir = regCase + "/" + timeFolder
    foldersFiles = pyOFLauncher.getFoldersFiles(wdir)
    regNames = foldersFiles[0]
    #region毎に処理
    for regName in regNames:
        regDir = wdir + "/" + regName
        for field in fields:
            fileName = regDir + "/" + field
            f = open(fileName)
            contents = f.readlines()
            f.close()
            #patchTypeをregionCouplingに変更
            i=0
            while i<len(contents)-2:
                n = string.find(contents[i], "_to_")
                m = string.find(contents[i+2], "zeroGradient")
                if n>=0 and m>=0:
                    contents[i+2] = checkCharas(contents[i+2],
                                    "zeroGradient", "regionCoupling")
                    i += 2
                i += 1
            #remoteFieldを追加
            i=0
            repContents = []
            while i<len(contents):
                repContents.append(contents[i])
                if string.find(contents[i], "regionCoupling") >= 0:
                    addLine = "        remoteField     " + field + ";\n"
                    repContents.append(addLine)
                i += 1
            #修正内容を書込
            f = open(fileName, "w")
            for line in repContents:
                f.write(line)
            f.close()

#
#  remakeBoundaryZeroGradient
#    各regionのbaundary（calculatedのみ）をzeroGradientに書き換える
def remakeBoundaryZeroGradient(regCaseDir, timeFolder):
    regNameDir = glob.glob(regCaseDir + "/" + timeFolder + "/*")
    regNames = []
    for name in regNameDir:
        regNames.append(name.split("/")[-1])
    for nameDir in regNameDir:
        fields = glob.glob(nameDir + "/*")
        for field in fields:
            if os.path.isfile(field) == True:
                contents = []
                f=open(field)
                for line in f.readlines():
                    contents.append(line)
                f.close()
                i=0
                while i<len(contents):
                    n = string.find(contents[i], "calculated")
                    if n>=0:
                        contents[i] = contents[i][:n] + "zeroGradient" + contents[i][n+len("calculated"):]
                    i=i+1
                f=open(field,"w")
                for line in contents:
                    f.write(line)
                f.close()
    print "\n各RegionのboundaryFieldをzeroGradientに修正しました。"

#
#  makeChangeDictionaryAllRegionsDict
#    changeDictionaryAllRegionsDictを作成する。
def makeChangeDictionaryAllRegionsDict(regCaseDir, timeFolder):
    regNameDir = glob.glob(regCaseDir + "/" + timeFolder + "/*")
    regNames = []
    for name in regNameDir:
        regNames.append(name.split("/")[-1])
    regNames.sort()
    c = []
    c.append(headerDict())
    for regName in regNames:
        field = "T"
        fieldDir = regCaseDir + "/" + timeFolder + "/" + regName + "/" + field
        patches = getPatches(fieldDir)
        repContents = replacementDict(regName, field, patches)
        c.append("\n" + regName + "\n")
        c.append("{\n")
        c.append(repContents)
        c.append("}\n")
        c.append("\n")
    c.append(footerDict())
    f=open(regCaseDir + "/system/changeDictionaryAllRegionsDict", "w")
    for line in c:
        f.write(line)
    f.close()
    print "\nsystemフォルダ内に「changeDictionaryAllRegionsDict」を作成しました。"


def getPatches(fieldDir):
    regName = fieldDir.split("/")[-2]
    f=open(fieldDir)
    contents = []
    for line in f.readlines():
        contents.append(line)
    f.close()
    patches = []
    for line in contents:
        n = string.find(line, regName + "_to_")
        if n>=0:
            patch = line.split()[0]
            patches.append(patch)
    return patches

def replacementDict(regName, field, patches):
    c=""
    c=c+    "dictionaryReplacement\n"
    c=c+    "{\n"
    c=c+    "    " + field + "\n"
    c=c+    "    {\n" 
    c=c+    "        boundaryField\n"
    c=c+    "        {\n"
    for patch in patches:
        c=c+"            " + patch + "\n"
        c=c+"            {\n"
        c=c+"                type            solidWallMixedTemperatureCoupled;\n"
        c=c+"                neighbourFieldName  " + field + ";\n"
        c=c+"                K               K;\n"
        c=c+"                value           uniform 0;\n"
        c=c+"            }\n"
    c=c+    "        }\n"
    c=c+    "    }\n"
    c=c+    "}\n"
    return c


def headerDict():
    header = setFoamContents.makeFoamHeader()
    foamFile = setFoamContents.makeFoamFile("2.0", "ascii", "dictionary", "system", "changeDictionaryAllRegionsDict")
    header = header + foamFile
    return header


def footerDict():
    footer = setFoamContents.makeFooter()
    return footer

#
#  copyFvschemesFvsolution
#    各regionにfvSchemes, fvSolutionをコピーする
def copyFvschemesFvsolution(regCaseDir, timeFolder):
    regNameDir = glob.glob(regCaseDir + "/" + timeFolder + "/*")
    regNames = []
    for name in regNameDir:
        regNames.append(name.split("/")[-1])
    regNames.sort()
    for regName in regNames:
        source = regCaseDir + "/system/fvSchemes"
        copy = regCaseDir + "/system/" + regName + "/fvSchemes"
        shutil.copyfile(source, copy)
        source = regCaseDir + "/system/fvSolution"
        copy = regCaseDir + "/system/" + regName + "/fvSolution"
        shutil.copyfile(source, copy)
    print "\nfvSchemes, fvSolutionを各regionにコピーしました。"

#
#  createRegCase
#    multiRegion用、conjugate用にcaseを作成
def createRegCase(flag):
    print "出来上がったregionを整形します。"
    caseDir = os.getcwd()
    timeFolders = pyOFLauncher.getTimeFolders(caseDir)
    if len(timeFolders) == 2:
        regCaseDir = caseDir + "/regCase"
        if len(glob.glob(regCaseDir)) != 0:
            shutil.rmtree(regCaseDir)
        os.mkdir(regCaseDir)
        src = caseDir + "/system"
        dst = regCaseDir + "/system"
        shutil.copytree(src,dst)
        src = caseDir + "/constant"
        dst = regCaseDir + "/constant"
        shutil.copytree(src,dst)
        src = caseDir + "/" + timeFolders[1]
        dst = regCaseDir + "/" + timeFolders[0]
        shutil.copytree(src,dst)
        shutil.rmtree(src)
        #regNameのフォルダをconstant内に作成
        makeRegNameIntoConstant(regCaseDir, timeFolders[0])
        #boundaryファイルのpatchTypeを修正
        remakeBoundaryFile(regCaseDir, timeFolders[0])
        #各RegionのboundaryFieldをzeroGradientに変更
        remakeBoundaryZeroGradient(regCaseDir, timeFolders[0])
        #multiRegion用に修正
        if flag == "reg":
            #changeDictionaryAllRegionsDictを作成
            makeChangeDictionaryAllRegionsDict(regCaseDir, timeFolders[0])
        #各regionにfvSchemes,fvSolutionをコピーする。
        copyFvschemesFvsolution(regCaseDir, timeFolders[0])
        error = ["OK", ""]
    else:    
        msg = "error: 時間フォルダがないか、領域分割できていません。"
        error = ["NG", msg]
        print msg 
    return error


#  conjugate用にcaseを作成
def RunConjugate():
    splitMeshRegions()
    error = createRegCase("conj")
    return error

#  multiRegion用にcaseを作成
def Run():
    splitMeshRegions()
    error = createRegCase("reg")
    return error

if __name__ == "__main__":
    splitMeshRegions()
    createBackupFile()

